/*

======================================================================

  Seyon is Copyright (c) 1992 of Muhammad M. Saggaf. Seyon is not
  public domain. Permission is granted to use and distribute Seyon
  freely for any use and to sell it at any price without reference to
  the copyright owner provided that in all above cases Seyon is intact
  and is not made part of any program either in whole or in part and
  that this copyright notice is included with Seyon. Permission is
  also granted to modify the source as long as the modified source is
  not distributed.

======================================================================

*/

#include <stdio.h>
#include <signal.h>
#include <setjmp.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Command.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/Viewport.h>

#include "MultiList.h"

#include "seyon.h"
#include "SeDecl.h"
#include "SeDeclX.h"

extern Widget SeCreatePopupTopLevel(), SeCreatePopup(), SeCreateForm();
extern Widget SeAddLabel(), SeAddDialog();
extern void RestartTerm();
extern char *get_word(), *read_line();

extern XtAppContext app_con;
extern int script_flag;
extern Boolean sevenBitMaskDefault;

void do_dial(), DismissDirectory(), DialerEnd(), dial_timer_handler();
void CancelDialer(), CancelDialerAndHangup(), CleanUpAndStartTerm();
void ClearItems(), dialer_terminate(), ManualDial(), DoManualDial();
void ReReadPhoneList();

char *script_file;
char phone_number[SM_BUF] = "", dialMsg[SM_BUF];;

short dialTime, dialTry;
static pid_t dial_pid;
jmp_buf dial_env;

Boolean directory_is_up = False, dialing_in_progress = False, 
  manual_dial = False;
XfwfMultiListWidget mlw;
int current_item;

/* 
 * the top routine for the dialing directory
 */

void top_dial(w)
	 Widget w;
{
  Widget form, view, list, widget;
  static Widget popup;
  static char phoneFile[REG_BUF];
  static String hosts[MAX_ENT + 1] = { NULL };
  Arg args[1];
 
  if (directory_is_up) {
	SeBeep(w);
	return;
  }
  else if (hosts[0] == NULL) {
	if (read_seyon_file(qres.phoneFile, phoneFile, hosts) == 0)
	  return;
  } 
  else {
	directory_is_up = True;
	XtPopup(popup, XtGrabNone);
	return;
  }

  directory_is_up = True;

  popup = SeCreateTopLevelPopup("directory", w, NULL, 
								XtParent(XtParent(w)));
  form = SeAddForm("form", popup);
  view = XtCreateManagedWidget("view", viewportWidgetClass, form, NULL, 0);
  
  XtSetArg(args[0], XtNlist, hosts);
  list = XtCreateManagedWidget("list", xfwfMultiListWidgetClass, 
							   view, args, 1);
  mlw = (XfwfMultiListWidget) list;

  SeSetViewportDimFromMultiList(view, list, 10);

  XtAddCallback(list, XtNcallback, do_dial, NULL);

  SeAddButton("dismiss", form, DismissDirectory);
  SeAddButtonWithClientData("edit", form, SeEditFile, (XtPointer) phoneFile);
  SeAddButtonWithClientData("reread", form, ReReadPhoneList, 
							(XtPointer) hosts);
  SeAddButton("manual", form, ManualDial);
  SeAddButton("ok", form, do_dial);
  SeAddButton("clear", form, ClearItems);
  SeAddButton("cancel", form, CancelDialer);
  SeAddButton("hangup", form, CancelDialerAndHangup);

  XtPopup(popup, XtGrabNone);
} 

/* 
 * gets the first selected item and dials it
 */

void do_dial()
{
  static XfwfMultiListReturnStruct *item;
  String item_string;
  Boolean Bdummy;

  if (dialing_in_progress) {
	SeBeep(mlw);
	return;
  }

  item = XfwfMultiListGetHighlighted(mlw);
  
  if (item->num_selected == 0) {
	SeBeep(mlw);
	return;
  }

  current_item = 0;

  if (XfwfMultiListGetItemInfo(mlw, item->selected_items[current_item], 
							   &item_string, &Bdummy, &Bdummy) == False)
	{
	  se_error("could not get dial list info");
	  return;
	}

  dialing_in_progress = True;
  manual_dial = False;

  get_dial_info(item_string);
  kill_terminal();
  dialTry = 1;
  exec_dial();
}

/* 
 * executes the dialing as a forked process and communicates progress
 * to the main process 
 */

exec_dial()
{
  if ((dial_pid = se_fork()) == 0) {

	signal(SIGTERM, dialer_terminate);

	if(dial_number(phone_number) == 0) {
	  write_child_info(child_pipe, DIAL_CIRCULATE, NULL);
	  put_parameters();
	  exit(1);
	}
								/*  let the user know of the connection */
	write_child_info(child_pipe, REMOVE_CURRENT_ITEM, "CONNECTED");

	if(script_flag) {
	  write_child_info(child_pipe, SET_MESSAGE, "RUNNING SCRIPT");
	  do_script(script_file);
	}

	write_child_info(child_pipe, CLEANUP_AFTER_DIALER, "CONNECTION COMPLETE");
	put_parameters();
	exit(0);
  } 
}

/* 
 * signal handler for canceling the dialing process
 */

void dialer_terminate()
{
  signal(SIGTERM, SIG_IGN);
  put_parameters();
  exit(1);
}

/* 
 * updates the dialing directory from teh phone list file
 */

void ReReadPhoneList(widget, hosts)
	 Widget widget;
	 XtPointer hosts;
{
  Widget form;
  char *ptr;

  if (dialing_in_progress) {
	SeBeep(widget);
	return;
  }

  directory_is_up = False;
  form = XtParent(widget);
  DestroyPopupPrompt(NULL, form, NULL);  

  for(ptr = (char*) hosts; *ptr; ptr++)
	free(ptr);
  ((String*) hosts)[0] = NULL;

  top_dial(XtParent(XtParent(form)));
}

/* 
 * sets up the manual dialing popup
 */

void ManualDial(widget)
	 Widget widget;
{
  Widget  popup, dialog;

  if (dialing_in_progress) {
	SeBeep(widget);
	return;
  }

  popup = SeCreatePopup("shell", widget);
  dialog = SeAddDialog("dialog", popup);

  SeSetDialogValue(dialog, phone_number);  

  XawDialogAddButton(dialog, "ok", DoManualDial, NULL);
  XawDialogAddButton(dialog, "cancel", DestroyPopupPrompt,
					 (XtPointer) dialog);

  XtPopup(popup, XtGrabExclusive);
}

/* 
 * does manual dialing
 */

void DoManualDial(widget)
	 Widget widget;
{
  Widget dialog = XtParent(widget);

  script_flag = 0;
  strcpy(phone_number, XawDialogGetValueString(dialog));

  DestroyPopupPrompt(NULL, (XtPointer) dialog, (XtPointer) NULL);  

  dialing_in_progress = True;
  manual_dial = True;

  kill_terminal();
  dialTry = 1;
  exec_dial();
}

/* 
 * action proc for manual dialing
 */

void manual_dial_action_ok(widget)
	 Widget widget;
{
  DoManualDial(widget);
}  

/* 
 * dismisses the dialing directory
 */

void DismissDirectory(widget)
	 Widget widget;
{
/*  if (dialing_in_progress) {
	SeBeep(widget);
	return;
  }*/

  directory_is_up = False;
  XtPopdown(XtParent(XtParent(widget)));
}

/* 
 * clears all selected items 
 */

void ClearItems()
{
  XfwfMultiListUnhighlightAll(mlw);
}

/* 
 * unhighlights (unselects) the current item
 */

RemoveCurrentItem()
{
  static XfwfMultiListReturnStruct *item;

  item = XfwfMultiListGetHighlighted(mlw);
  XfwfMultiListUnhighlightItem(mlw, item->selected_items[current_item]);
}

/* 
 * circulates to the next selected item and dials it
 */

DialCirculate()
{
  static XfwfMultiListReturnStruct *item;
  String item_string;
  Boolean Bdummy;

  if(manual_dial)
	dialTry++;
  else {
	get_parameters();
	
	item = XfwfMultiListGetHighlighted(mlw);
	if (++current_item == item->num_selected ) {
	  current_item = 0;
	  dialTry++;
	}

	if (XfwfMultiListGetItemInfo(mlw, item->selected_items[current_item], 
							   &item_string, &Bdummy, &Bdummy) == False)
	  {
		se_error("could not get dial list info");
		return;
	  }

	get_dial_info(item_string);
  }

  if (dialTry > qres.dialRepeat) {
	SeyonMessage("Connection Failed");
	CleanUpAndStartTerm();
	return;
  }
  
  exec_dial();
}

/* 
 * this routine is executed when dialing ends
 */

void DialerEnd()
{
  get_parameters();
  CleanUpAndStartTerm();
}

/* 
 * cancels dialing 
 */

void CancelDialer()
{
  if (!dialing_in_progress) {
	SeBeep(mlw);
	return;
  }

  kill(dial_pid, SIGTERM);
  
  cancel_dial(1);
  while(wait((int *) 0) >= 0);	/* wait for the child to die */

  DialerEnd();
}

/* 
 * similar to the above, but also hangs up
 */

void CancelDialerAndHangup()
{
  if (!dialing_in_progress) {
	SeBeep(mlw);
	return;
  }

  CancelDialer();
  hangup();
}

/* 
 * cleans up and starts the terminal process
 */

void CleanUpAndStartTerm()
{
  dialing_in_progress = False;

  start_terminal();
}

/* 
 * gets the dialing info (number, bits, ..etc) for an item and sets the
 * required parameters
 */

get_dial_info(item)
	 char *item;
{
  static char sbuf[REG_BUF];
  char *buf, *nbr, *ptr, *ptr1, c;

  buf = strcpy(sbuf, item);

  if (ptr = strstr(buf, "BPS=")) {
	ptr += 4;
	if (mbaud(ptr) == -1) 
	  showf("<< Invalid BPS value: %s >>", ptr);
  } 
  else 
	mbaud(qres.defaultBPS);
  
  if (ptr = strstr(buf, "BITS=")) {
	ptr += 5;
	switch (*ptr) {
	case '8':
	  qres.sevenBitMask = False;
	  break;
	case '7':
	  qres.sevenBitMask = True;
	  break;
	default:
	  se_warning("invalid BITS value in dialing directory");
	}
  }
  else
	qres.sevenBitMask = sevenBitMaskDefault;
  
  while (isspace(*buf) && *buf)
	buf++;
  
  if (!(*buf))
	return;
  
  for (nbr = buf; !isspace(*buf) && *buf; buf++);
  
  c = *buf;
  *buf = '\0';
  strcpy(phone_number, nbr);

  *buf = c;
  if (ptr = strstr(buf, "SCRIPT=")) {
	ptr += 7;
	ptr1 = ptr;
	while (!isspace(*ptr1) && *ptr1 != '\0')
	  ptr1++;
    *ptr1 = '\0';

	script_file = ptr;
	script_flag = 1;
	linkflag = TRUE;
  }
  
  else
	script_flag = 0;
}


/* 
 * this routine actually does the dialing
 */

#define DIALALARM 5

dial_number(number)
	 char *number;
{
  char *res, *msg;
  int i;
  
  if(setjmp(dial_env) != 0) {
	alarm(0);
	cancel_dial(0);

	writef_child_info(child_pipe, SET_MESSAGE, "SLEEPING FOR %d SECONDS...", 
					  qres.dialDelay);
	sleep(qres.dialDelay);

	return 0;
  }

  dialTime = qres.dialTimeOut;

  if (dialTry == 1)
    sprintf(dialMsg, "DIALING %s", number);
  else
    sprintf(dialMsg, "REDIALING %s (TRY #%d)", number, dialTry);

  writef_child_info(child_pipe, SET_MESSAGE, "%s... %d", dialMsg, dialTime);

  flush_modem();
  
  sendf_slowly("\r%s %s%s", qres.dialPrefix, number, qres.dialSuffix);

  signal(SIGALRM, dial_timer_handler);
  alarm(DIALALARM);

  while(1) {

	res = read_line();
	show(res);
	  
	msg = SSpc(qres.connectString);
	if (!strncmp(res, msg, strlen(msg))) {
	  alarm(0);
	  return 1;
	}

	for(i = 0; i < 3; i++) {
	  msg = SSpc(qres.noConnectString[i]);
	  if ((*msg) && (!strncmp(res, msg, strlen(msg))))
		longjmp(dial_env, 1);
	  }
  } /* end of while(1) */
}

/* 
 * alarm handler for the dialing timeout
 */

void dial_timer_handler()
{
  dialTime -= DIALALARM;
  writef_child_info(child_pipe, SET_MESSAGE, "%s... %d", dialMsg, dialTime);
  
  if (dialTime > 0) {
	signal(SIGALRM, dial_timer_handler);
	alarm(DIALALARM);
  } else {
    signal(SIGALRM, SIG_DFL);
    alarm(0);
	longjmp(dial_env, 1);
  }
}
	
