/*

======================================================================

  Seyon is Copyright (c) 1992 of Muhammad M. Saggaf. Seyon is not
  public domain. Permission is granted to use and distribute Seyon
  freely for any use and to sell it at any price without reference to
  the copyright owner provided that in all above cases Seyon is intact
  and is not made part of any program either in whole or in part and
  that this copyright notice is included with Seyon. Permission is
  also granted to modify the source as long as the modified source is
  not distributed.

======================================================================

*/

#include <stdio.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <X11/Xaw/Command.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/Label.h>

#include "seyon.h"
#include "SeDecl.h"
#include "SeDeclX.h"

#ifndef HELPFILE
#define HELPFILE "/usr/lib/X11/seyon.help"
#endif

extern void top_shell(), upload_acc_ok(), shell_acc_ok(), set_action_ok();
extern void manual_dial_action_ok();
extern void top_help(), top_misc(), FunMessage();
extern Boolean CvtStringToStringArray();

void top_hangup(), do_hangup(), top_exit(), exit_no_hangup(), exit_hangup();

Boolean sevenBitMaskDefault;
Widget statusMessage;

get_resources(topLevel)
	 Widget topLevel;
{
#define offset(field) XtOffsetOf(struct QueryResources, field)

  static XtResource resources[] = {
	{ "modem", "Modem", XtRString, sizeof(String), 
		offset(modem), XtRString, (XtPointer) NULL },
	{ "script", "Script", XtRString, sizeof(String), 
		offset(script), XtRString, (XtPointer) NULL },

	{ "defaultBPS", "DefaultBPS", XtRString, sizeof(String), 
		offset(defaultBPS), XtRString, (XtPointer) "9600" },
	{ "sevenBitMask", "SevenBitMask", XtRBoolean, sizeof(Boolean), 
		offset(sevenBitMask), XtRImmediate, (XtPointer) False },
	{ "newlineTranslation", "NewlineTranslation", XtRBoolean, 
		sizeof(Boolean), offset(newlineTranslation), XtRImmediate, 
		(XtPointer) True },
	{ "backspaceTranslation", "BackspaceTranslation", XtRBoolean, 
		sizeof(Boolean), offset(backspaceTranslation), XtRImmediate,
		(XtPointer) False },
	{ "metaKeyTranslation", "MetaKeyTranslation", XtRBoolean, 
		sizeof(Boolean), offset(metaKeyTranslation), XtRImmediate,
		(XtPointer) True },
	{ "xonxoffFlowControl", "XonxoffFlowControl", XtRBoolean, 
		sizeof(Boolean), offset(xonxoffFlowControl), XtRImmediate,
		(XtPointer) False },
	{ "rtsctsFlowControl", "RtsctsFlowControl", XtRBoolean, 
		sizeof(Boolean), offset(rtsctsFlowControl), XtRImmediate,
		(XtPointer) False },

	{ "dialPrefix", "DialPrefix", XtRString, sizeof(String), 
		offset(dialPrefix), XtRString, (XtPointer) "ATDT" },
	{ "dialSuffix", "DialSuffix", XtRString, sizeof(String), 
		offset(dialSuffix), XtRString, (XtPointer) "\r" },
	{ "dialCancel", "DialCancel", XtRString, sizeof(String), 
		offset(dialCancel), XtRString, (XtPointer) "\r" },

	{ "dialTimeOut", "DialTimeOut", XtRShort, sizeof(short), 
		offset(dialTimeOut), XtRImmediate, (XtPointer) 45 },
	{ "dialDelay", "DialDelay", XtRShort, sizeof(short), 
		offset(dialDelay), XtRImmediate, (XtPointer) 10 },
	{ "dialRepeat", "DialRepeat", XtRShort, sizeof(short), 
		offset(dialRepeat), XtRImmediate, (XtPointer) 5 },

	{ "connectString", "ConnectString", XtRString, sizeof(String), 
		offset(connectString), XtRString, (XtPointer) "CONNECT" },
	{ "noConnectString1", "NoConnectString1", XtRString, sizeof(String), 
		offset(noConnectString[0]), XtRString, (XtPointer) "NO CARRIER" },
	{ "noConnectString2", "NoConnectString2", XtRString, sizeof(String), 
		offset(noConnectString[1]), XtRString, (XtPointer) "NO DIALTONE" },
	{ "noConnectString3", "NoConnectString3", XtRString, sizeof(String), 
		offset(noConnectString[2]), XtRString, (XtPointer) "BUSY" },
	{ "noConnectString4", "NoConnectString4", XtRString, sizeof(String), 
		offset(noConnectString[3]), XtRString, (XtPointer) "VOICE" },

	{ "hangupConfirm", "HangupConfirm", XtRBoolean, sizeof(Boolean), 
		offset(hangupConfirm), XtRImmediate, (XtPointer) True},
	{ "exitConfirm", "ExitConfirm", XtRBoolean, sizeof(Boolean), 
		offset(exitConfirm), XtRImmediate, (XtPointer) True},

	{ "zmodemAutoDownload", "ZmodemAutoDownload", XtRBoolean, 
		sizeof(Boolean), offset(zmodemAutoDownload), XtRImmediate, 
		(XtPointer) True},
	{ "zmodemAutoDownloadCommand", "ZmodemAutoDownloadCommand", 
		XtRString, sizeof(String), offset(zmodemAutoDownloadCommand), 
		XtRString, (XtPointer) "$rz" },

	{ "defaultDirectory", "DefaultDirectory", XtRString, sizeof(String),
		offset(defaultDirectory), XtRString, (XtPointer) "~/.seyon" },
	{ "scriptDirectory", "scriptDirectory", XtRString, sizeof(String),
		offset(scriptDirectory), XtRString, (XtPointer) NULL },
	{ "startupFile", "StartupFile", XtRString, sizeof(String),
		offset(startupFile), XtRString, (XtPointer) "startup" },
	{ "phoneFile", "PhoneFile", XtRString, sizeof(String),
		offset(phoneFile), XtRString, (XtPointer) "phonelist" },
	{ "protocolsFile", "ProtocolsFile", XtRString, sizeof(String),
		offset(protocolsFile), XtRString, (XtPointer) "protocols" },
	{ "captureFile", "CaptureFile", XtRString, sizeof(String),
		offset(captureFile), XtRString, (XtPointer) "capture" },
	{ "helpFile", "HelpFile", XtRString, sizeof(String),
		offset(helpFile), XtRString, (XtPointer) HELPFILE },

	{ "funMessagesInterval", "FunMessagesInterval", XtRInt,	sizeof(int),
		offset(funMessagesInterval), XtRImmediate, (XtPointer) 60},
	{ "funMessages", "FunMessages", XtRStringArray, sizeof(String*),
		offset(funMessages), XtRStringArray, (XtPointer) NULL},

	{ "version", "Version", XtRString, sizeof(String), 
		offset(version), XtRString, (XtPointer) NULL },
  };  
  
#undef offset

  XtSetTypeConverter(XtRString, XtRStringArray, CvtStringToStringArray,
					 NULL, 0, XtCacheNone, NULL);
  
  XtGetApplicationResources(topLevel, (XtPointer) &qres, resources,
							XtNumber(resources), NULL, 0);
}

init_variables()
{
  char buffer[REG_BUF];

  if(qres.scriptDirectory == NULL)
	qres.scriptDirectory = qres.defaultDirectory;

  sprintf(captureFile, "%s/%s", expand_fname(SSpc(qres.defaultDirectory), 
											 buffer), qres.captureFile);

  sevenBitMaskDefault = qres.sevenBitMask;
}

create_command_center(app_con, topLevel)
	 XtAppContext app_con;
	 Widget topLevel;
{
  Widget mainBox, menuBox, statusBox;
  struct command *ptr;

  static XtActionsRec actionTable[] = {
    {"UploadOk",     upload_acc_ok},
    {"ShellOk",      shell_acc_ok},
	{"SetOk",        set_action_ok},
	{"ManualDialOk", manual_dial_action_ok}
  };

  static struct command cmd[] = {
	{ "help",         top_help },
	{ "set",          top_set },
	{ "dial",         top_dial },
	{ "transfer",     top_transfer },
	{ "shellCommand", top_shell },
	{ "misc",         top_misc },
	{ NULL,           NULL }
  };
  
  XtAppAddActions(app_con, actionTable, XtNumber(actionTable));

  mainBox = SeAddForm("mainBox", topLevel);
  menuBox = SeAddForm("menuBox", mainBox);
  statusBox = SeAddForm("statusBox", mainBox);
  
  for (ptr = cmd; ptr->name != NULL; ptr++)
	SeAddButton(ptr->name, menuBox, ptr->call_back);

  if(qres.hangupConfirm)
	SeAddButton("hangup", menuBox, top_hangup);
  else
	SeAddButton("hangup", menuBox, hangup);

  if(qres.exitConfirm)
	SeAddButtonWithClientData("exit", menuBox, top_exit, 
							  (XtPointer) topLevel);
  else
	SeAddButtonWithClientData("exit", menuBox, s_exit,
							  (XtPointer) topLevel);

  SeAddLabel("statusTitle", statusBox);
  statusMessage = SeAddLabel("statusMessage", statusBox);

  if (qres.funMessages)
	XtAppAddTimeOut(app_con, qres.funMessagesInterval * 1000, FunMessage,
					(XtPointer) app_con);
}

/* 
 * displays the help file
 */

void top_help(parent)
	 Widget parent;
{
  SeDisplayFile(parent, qres.helpFile);
}

void top_hangup(w)
	 Widget w;
{
  Widget  popup, dialog;

  popup = SeCreatePopup("hangup", w);
  dialog = SeAddDialog("dialog", popup);

  XawDialogAddButton(dialog, "yes", do_hangup, (XtPointer) dialog);
  XawDialogAddButton(dialog, "cancel", DestroyParentPopup,
					 (XtPointer) dialog);

  XtPopup(popup, XtGrabExclusive);
}

void do_hangup(widget, client_data)
	 Widget widget;
	 XtPointer client_data;
{
  hangup();
  DestroyParentPopup(NULL, client_data, NULL);
}

void top_exit(w, client_data)
	 Widget w;
	 XtPointer client_data;
{
  Widget  popup, dialog;

  popup = SeCreatePopup("exit", w);
  dialog = XtCreateManagedWidget("dialog", dialogWidgetClass, popup,
								 NULL, 0);

  XawDialogAddButton(dialog, "yes", exit_hangup, client_data);
  XawDialogAddButton(dialog, "no", exit_no_hangup, client_data);
  XawDialogAddButton(dialog, "cancel", DestroyParentPopup,
					 (XtPointer) dialog);

  XtPopup(popup, XtGrabExclusive);
}

void exit_no_hangup(widget, client_data)
	 Widget widget;
	 XtPointer client_data;
{
  DestroyParentPopup(NULL, XtParent(widget), NULL);
  s_exit(widget, client_data);
}

void exit_hangup(widget, client_data)
	 Widget widget;
	 XtPointer client_data;
{
  hangup();
  DestroyParentPopup(NULL, XtParent(widget), NULL);
  s_exit(widget, client_data);
}
