/*

======================================================================

  Seyon is Copyright (c) 1992 of Muhammad M. Saggaf. Seyon is not
  public domain. Permission is granted to use and distribute Seyon
  freely for any use and to sell it at any price without reference to
  the copyright owner provided that in all above cases Seyon is intact
  and is not made part of any program either in whole or in part and
  that this copyright notice is included with Seyon. Permission is
  also granted to modify the source as long as the modified source is
  not distributed.

======================================================================

*/

#include "config.h"

#ifdef HAVE_TERMIOS
#include <termios.h>
#else
#ifdef HAVE_TERMIO
#include <termio.h>
#endif
#endif

int io_set_attr(fd, io)
	 int fd;

#ifdef HAVE_TERMIOS
	 struct termios *io;
{
  int res;

  res = tcsetattr(fd, TCSADRAIN, io);
  if ( res < 0)
	perror("tcsetattr");

  return res;

#else
#ifdef HAVE_TERMIO
     struct termio *io;
{
  int res;

  res = ioctl(fd, TCSETAW, io);
  if ( res < 0)
	perror("ioctl-set");

  return res;
#endif
#endif
}

int io_get_attr(fd, io)
	 int fd;

#ifdef HAVE_TERMIOS
	 struct termios *io;
{
  int res;

  res = tcgetattr(fd, io);
  if (res < 0)
	perror("tcgetattr");

  return res;

#else
#ifdef HAVE_TERMIO
     struct termio *io;
{
  int res;

  res = ioctl(fd, TCGETA, io);
  if (res < 0)
	perror("ioctl-get");

  return res;
#endif
#endif
}

io_flush(fd)
	 int fd;
{
  int res;

#ifdef HAVE_TERMIOS
  res = tcflush(fd, TCIOFLUSH);
  if (res < 0)
	perror("tcflush");

#else
#ifdef HAVE_TERMIO
  res = ioctl(fd, TCFLSH, 2);
  if (res < 0)
	perror("ioctl-flush");
#endif
#endif

  return res;
}

io_send_break(fd)
	 int fd;
{
  int res;

#ifdef HAVE_TERMIOS
  res = tcsendbreak(fd, 0);

#else
#ifdef HAVE_TERMIO
  res = ioctl(fd, TCSBRK, 0);
#endif
#endif

  return res;
}

io_set_speed(io, speed)

#ifdef HAVE_TERMIOS
	 struct termios *io;
	 speed_t speed;
{
  cfsetospeed(io, speed);
  cfsetispeed(io, speed);

#else
#ifdef HAVE_TERMIO
     struct termio *io;
	 speed_t speed;
{
  io->c_cflag &= ~CBAUD;
  io->c_cflag |= speed;
#endif
#endif
}

speed_t io_get_speed(io)

#ifdef HAVE_TERMIOS
	 struct termios *io;
{
  return cfgetospeed(io);

#else
#ifdef HAVE_TERMIO
     struct termio *io;
{
  return io->c_cflag & CBAUD;
#endif
#endif
}

