/*

======================================================================

  Seyon is Copyright (c) 1992 of Muhammad M. Saggaf. Seyon is not
  public domain. Permission is granted to use and distribute Seyon
  freely for any use and to sell it at any price without reference to
  the copyright owner provided that in all above cases Seyon is intact
  and is not made part of any program either in whole or in part and
  that this copyright notice is included with Seyon. Permission is
  also granted to modify the source as long as the modified source is
  not distributed.

======================================================================

*/

#include <stdio.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Dialog.h>

#include "seyon.h"
#include "SeDecl.h"
#include "SeDeclX.h"

extern Widget SeCreatePopup(), SeAddToggle();
extern char *str_strip_lead_end_space();

void Toggle7BitMask(), ToggleNewlineMode(), ToggleDelMode();
void ToggleMetaKeyTr();
void ToggleXOffMode(), ToggleRtsctsMode(), ToggleZmodemAutoDownload(); 
void SetBaud();
void UpdateStates();

struct set_toggle {
  Widget widget;
  char *name;
  int state;
  void (*call_back)();
};

struct set_value {
  Widget widget;
  char *name;
  char value[SM_BUF];
  void (*call_back)();
};

static  struct set_toggle set_toggle[] = {
  { NULL,    "7bit",      0,         Toggle7BitMask },
  { NULL,    "nl",        0,         ToggleNewlineMode },
  { NULL,    "del",       0,         ToggleDelMode },
  { NULL,    "meta_tr",   0,         ToggleMetaKeyTr },
  { NULL,    "xoff",      0,         ToggleXOffMode },
  { NULL,    "rtscts",    0,         ToggleRtsctsMode },
  { NULL,    "autozm",    0,         ToggleZmodemAutoDownload },
  { NULL,     NULL,       0,         NULL }
};
  
static struct set_value set_value[] = {
  { NULL,    "baud",      "",         SetBaud },
  { NULL,     NULL,       "",         NULL }
};

void top_set(w)
	 Widget w;
{
  Widget popup, form, widget;
  struct set_toggle *tptr;
  struct set_value *vptr;

  tptr = set_toggle;
  tptr->state = qres.sevenBitMask; tptr++;
  tptr->state = qres.newlineTranslation; tptr++;
  tptr->state = qres.backspaceTranslation; tptr++;
  tptr->state = qres.metaKeyTranslation; tptr++;
  tptr->state = qres.xonxoffFlowControl; tptr++;
  tptr->state = qres.rtsctsFlowControl; tptr++;
  tptr->state = qres.zmodemAutoDownload; tptr++;

  vptr = set_value;
  sprintf(vptr->value, "%d", mbaud((char *) 0)); vptr++;

  popup = SeCreatePopup("set", w);
  form = XtCreateManagedWidget("form", formWidgetClass, popup, NULL, 0);

  for (tptr = set_toggle; tptr->name != NULL; tptr++) {
	tptr->widget = SeAddToggle(tptr->name, form, NULL);
	SeSetUnsetToggle(tptr->widget, tptr->state);
  }  

  for (vptr = set_value; vptr->name != NULL; vptr++) {
	vptr->widget = SeAddDialog(vptr->name, form);
	SeSetDialogValue(vptr->widget, vptr->value);  
  }  

  widget = XtCreateManagedWidget("ok", commandWidgetClass, form, NULL, 0);
  XtAddCallback(widget, XtNcallback, UpdateStates, (XtPointer) form);

  SeAddButton("cancel", form, DestroyPopupPrompt);

  XtPopup(popup, XtGrabExclusive);
}

void UpdateStates(widget, client_data)
	 Widget widget;
	 XtPointer client_data;
{
  struct set_toggle *tptr;
  struct set_value *vptr;
  int state;
  String value;

  for (tptr = set_toggle; tptr->name != NULL; tptr++) {
	state = SeGetToggleState(tptr->widget);
	if (state != tptr->state)
	  (*tptr->call_back)();
  }

  for (vptr = set_value; vptr->name != NULL; vptr++) {
	value = str_strip_lead_end_space(XawDialogGetValueString(vptr->widget));
	if (strcmp(value, vptr->value))
	  (*vptr->call_back)(value);
  }
  
  DestroyPopupPrompt(NULL, client_data, NULL);
  restart_terminal();

  SeyonMessage("PARAMETERS UPDATED");
}

void set_action_ok(widget)
	 Widget widget;
{
  UpdateStates(widget, (XtPointer) XtParent(XtParent(widget)));
}

void Toggle7BitMask()
{
  toggle_flag(&(qres.sevenBitMask));
}

void ToggleNewlineMode()
{
  toggle_flag(&(qres.newlineTranslation));
}

void ToggleDelMode()
{
  toggle_flag(&(qres.backspaceTranslation));
}

void ToggleMetaKeyTr()
{
  toggle_flag(&(qres.metaKeyTranslation));
}

void ToggleXOffMode()
{
  toggle_flag(&(qres.xonxoffFlowControl));
  xc_setflow();
}

void ToggleRtsctsMode()
{
  toggle_flag(&(qres.rtsctsFlowControl));
  set_rtscts();
}

void ToggleZmodemAutoDownload()
{
  toggle_flag(&(qres.zmodemAutoDownload));
}

void SetBaud(baud)
	 String baud;
{
  if (mbaud(baud) < 0)
	showf("<< Unsupported baud rate %s >>", baud);
}

void s_set_7bit();
void s_set_xoff(), s_set_baud();
void s_set_cr(), s_set_del(), set_meta_tr();
void s_set_rtscts(), s_set_autozm();

struct kw setlist[] = {
	{ "7bit",	   s_set_7bit },
	{ "nl",		   s_set_cr },
	{ "del",	   s_set_del },
	{ "meta_tr",   set_meta_tr },
	{ "xoff",	   s_set_xoff },
	{ "rtscts",	   s_set_rtscts },
	{ "autozm",	   s_set_autozm },
	{ "baud",	   s_set_baud },
	{ NULL,		   NULL }};

void s_set()
{
  struct kw *ptr;
  
  getword();
  
  if (word[0] == '\0') {
	fprintf(tfp, "SET keyword requires an argument\r\n");
	eof_flag++;
	return;
  }
  
  lc_word(word);
  
  for (ptr = setlist; ptr->keyword != NULL; ptr++)
	if (strcmp(ptr->keyword, word) == 0) {
	  (*ptr->rtn)();
	  return;
	}
  
  showf("<< Invalid SET keyword: %s >>", word);
  eof_flag++;
}

static char *statfmt = "%-8s %30s is %s\r\n";

void s_set_7bit()
{
  set_onoff(&(qres.sevenBitMask));
}

void s_set_cr()
{
  set_onoff(&(qres.newlineTranslation));
}

void s_set_del()
{
  set_onoff(&(qres.backspaceTranslation));
}

void set_meta_tr()
{
  set_onoff(&(qres.metaKeyTranslation));
}

void s_set_xoff()
{
  set_onoff(&(qres.xonxoffFlowControl));
  xc_setflow();
}

void s_set_rtscts()
{
  set_onoff(&(qres.rtsctsFlowControl));
  set_rtscts();
}

void s_set_autozm()
{
  set_onoff(&(qres.zmodemAutoDownload));
}

set_onoff(flag)
 Boolean *flag;
{
  char *strdup();
  char *ptr = strdup(word);
  
  getword();
  lc_word(word);
  
  if (strcmp(word, "on") == 0) {
	*flag = True;
  } else if (strcmp(word, "off") == 0) {
	*flag = False;
  } else {
	fprintf(tfp, "Set '%s' value must be 'on' or 'off'.\r\n", ptr);
	eof_flag++;
  }
    
  if (ptr != NULL)
	free(ptr);
}

void s_set_baud()
{
  getword();
  if (word[0] == '\0') {
	fprintf(tfp, "Set Baud must have a baud rate.\r\n");
	eof_flag++;
	return;
  }
  
  if (mbaud(word)<0) {
	fprintf(tfp, "Unsupported baud rate %s\r\n", word);
	eof_flag++;
  } 
  else if (!scriptflag)
	fprintf(tfp, "<<Baud rate set to %d>>\r\n", mbaud((char *) 0));
}

struct param {
  Boolean sevenBitMask;
  Boolean newlineTranslation;
  Boolean backspaceTranslation;
  Boolean metaKeyTranslation;
  Boolean xonxoffFlowControl;
  Boolean rtsctsFlowControl;
  Boolean zmodemAutoDownload;
};

put_parameters()
{
  struct param param;

  param.sevenBitMask = qres.sevenBitMask;
  param.newlineTranslation = qres.newlineTranslation;
  param.backspaceTranslation = qres.backspaceTranslation;
  param.metaKeyTranslation = qres.metaKeyTranslation;
  param.xonxoffFlowControl = qres.xonxoffFlowControl;
  param.rtsctsFlowControl = qres.rtsctsFlowControl;
  param.zmodemAutoDownload = qres.zmodemAutoDownload;
  
  if(write(child_pipe[1], (char *) &param, sizeof(param)) == -1)
	se_warning("could not write to pipe");
}  

get_parameters()
{
  struct param param;

  read(child_pipe[0], &param, sizeof(param));

  qres.sevenBitMask = param.sevenBitMask;
  qres.newlineTranslation = param.newlineTranslation;
  qres.backspaceTranslation = param.backspaceTranslation;
  qres.metaKeyTranslation = param.metaKeyTranslation;
  qres.xonxoffFlowControl = param.xonxoffFlowControl;
  qres.rtsctsFlowControl = param.rtsctsFlowControl;
  qres.zmodemAutoDownload = param.zmodemAutoDownload;

  sync_modem_attr();
}

