/*

======================================================================

  Seyon is Copyright (c) 1992 of Muhammad M. Saggaf. Seyon is not
  public domain. Permission is granted to use and distribute Seyon
  freely for any use and to sell it at any price without reference to
  the copyright owner provided that in all above cases Seyon is intact
  and is not made part of any program either in whole or in part and
  that this copyright notice is included with Seyon. Permission is
  also granted to modify the source as long as the modified source is
  not distributed.

======================================================================

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

#include "seyon.h"
#include "SeDecl.h"

char *str_strip_lead_end_space(), *str_strip_lead_space();
char *str_strip_end_space();
void shell_exec();

/* 
 * string routines
 */

/* 
 * replace unwanted characters with something else
 */

char *str_filter(str, unwanted, replacement)
	 char *str, *unwanted;
	 char replacement;
{
  int i, j;

  for (i=0; str[i]; i++)
	for (j=0; unwanted[j]; j++)
	  if (str[i] == unwanted[j])
		str[i] = replacement;

  return str;
}

/* 
 * strip spaces from both ends of a string
 */

char *str_strip_lead_end_space(str)
{
  return str_strip_lead_space(str_strip_end_space(str));
}

/* 
 * strip spaces from the beginning a string
 */

char *str_strip_lead_space(str)
	 char *str;
{
  int i;
  
  for(i = 0; isspace(str[i]); i++);
  return str + i;
}

/* 
 * strip spaces from the end a string
 */

char *str_strip_end_space(str)
	 char *str;
{
  int i;
  
  for(i = strlen(str) - 1; isspace(str[i]) && i; i--);
  str[++i] = '\0';

  return str;
}

char *str_strip_end_char(str)
     char *str;
{
  int length;
  
  length = strlen(str);
  str[length-1] = '\0';

  return str;
}

#ifndef HAVE_STRSTR

/*
 * find first occurance of str2 in str1 and return a pointer to it
 */

char *strstr(str1, str2)
	 char *str1, *str2;
{
  char *Sptr, *Tptr;
  int len = strlen(str1) - strlen(str2) + 1;
  
  if (*str2)
	for (; len > 0; len--, str1++) {
	  if (*str1 != *str2)
		continue;
	  
	  for (Sptr = str1, Tptr = str2; *Tptr != '\0'; Sptr++, Tptr++)
		if (*Sptr != *Tptr)
		  break;
	  
	  if (*Tptr == '\0')
		return str1;
	}
  
  return NULL;
}
#endif

char *strsqtok(str)
	 char *str;
{
  char *line, *wrd;
  static char *saved_ptr;

  if (str == NULL)
	line = saved_ptr;
  else
	line = str;

  while (isspace(*line) && *line)
	line++;
  
  if (*line == '\0')
	return NULL;
  else if (*line == '\"')
	for (wrd = ++line; *line != '\"'  && *line; line++);
  else
	for (wrd = line; !isspace(*line) && *line; line++);

  if(*line == '\0')
	saved_ptr = line;
  else
	saved_ptr = line+1;

  *line = '\0';
  return wrd;
}

/* 
 * return the first word of a string and set the start of the string
 * to point to the next one
 */

/* 
 * WARNING: words returned by this routine are not null-termnated
 */

char *str_parse(buf)
	 char **buf;
{
  char *line, *wrd;

  /* 
   * Remember: *buf is a pointer to the string, **buf is a character
   * in the string
   */

  line = *buf;

  while (isspace(*line) && *line)
	line++;
  
  if (!(*line))
	return NULL;
  else if (*line == '\"') {
	for (wrd = ++line; *line != '\"'  && *line; line++);
	*line = '\0';
	line++;
  }
  else
	for (wrd = line; !isspace(*line) && *line; line++);

  *buf = line;
  return wrd;
}

/* 
 * this routine is not currently used, and I'm not if it works
 */

char *get_word(str, word)
	 char *str, *word;
{
  char *wrd, c;

  while (isspace(*str) && *str)
	str++;
  
  if (!(*str)) 
	word[0] = '\0';

  else if (*str == '\"') {
	for (wrd = ++str; *str != '\"'  && *str; str++);
	*str = '\0';
	strcpy(word, wrd);
	*str = '\"';
	str++;
  }

  else {
	for (wrd = str; !isspace(*str) && *str; str++);
	c = *str;
	*str = '\0';
	strcpy(word, wrd);
	*str = c;
  }

  return str;
}
  
/*
 * miscellaneous routines
 */

/* 
 * a simple toggle
 */

void toggle_flag(flag)
	 int *flag;
{
  *flag = !*flag;
}

/*
 * print a message to the tty
 */

void show(msg)
	 char *msg;
{
  fprintf(tfp,"%s\r\n", msg);
}

/*
 * print a formatted message to the tty
 */

void showf(a, b, c, d)
	 char *a, *b, *c, *d;
{
  fprintf(tfp, a, b, c, d);
  fprintf(tfp, "\r\n");
}

se_error(msg)
	 char *msg;
{
  char buf[REG_BUF];

  sprintf(buf, ">> Error: %s.", msg);
  show(buf);
}

se_errorf(a, b, c, d)
	 char *a, *b, *c, *d;
{
  char buf[REG_BUF];
  
  sprintf(buf, a, b, c, d);
  se_error(buf);
}

se_warning(msg)
	 char *msg;
{
  char buf[REG_BUF];

  sprintf(buf, ">> Warning: %s.", msg);
  show(buf);
}

se_warningf(a, b, c, d)
	 char *a, *b, *c, *d;
{
  char buf[REG_BUF];
  
  sprintf(buf, a, b, c, d);
  se_warning(buf);
}

se_notice(msg)
	 char *msg;
{
  char buf[REG_BUF];

  sprintf(buf, ">> Notice: %s.", msg);
  show(buf);
}

se_noticef(a, b, c, d)
	 char *a, *b, *c, *d;
{
  char buf[REG_BUF];
  
  sprintf(buf, a, b, c, d);
  se_notice(buf);
}

se_perror(msg)
	 char *msg;
{
  char buf[REG_BUF] = ">> Error: ";

  perror(strcat(buf, msg));
}

se_perrorf(a, b, c, d)
	 char *a, *b, *c, *d;
{
  char buf[REG_BUF];

  sprintf(buf, a, b, c, d);
  se_perror(buf);
}

/* 
 * routines that have to do with forking or executing external commands
 */

/* 
 * fork a process and print an error message if forking is not successful
 */

se_fork()
{
  pid_t pid;

  pid = fork();

  if (pid < 0)
	show("<< Error: faild to fork process >>");

  return pid;
}  

/* 
 * execute an external command though a shell
 */
  
exec_cmd(command)
	  char *command;
{
  kill_terminal();
  shell_exec(command);
  start_terminal();
 }

void shell_exec(command)
	 char *command;
{
  static char *shell = NULL;
  char *scmd;
  pid_t child;
  int fio;

  if(command == NULL)
	return;

  if (!shell) {
	shell = getenv("SHELL");
	if (!shell)
	  shell = "/bin/sh";
  }

  io_set_attr(tfd, &oldmode);
  fio = get_modem_fio();
  
  if ((child = se_fork()) == 0) {
	scmd = str_strip_lead_end_space(command);

	if(*scmd == '$') {
	  se_notice("redirecting stdin/stdout");
	  mattach(); 				/* Attach modem to stdin, stdout */
	  scmd++;
	}

	if(*scmd == '\0') {
	  se_noticef("executing the shell '%s'", shell);
	  execl(shell, shell, (char*) NULL);
	  se_errorf("execution of '%s' failed", shell);
	  exit(1);
	}

	se_noticef("executing the command '%s'", scmd); 
	execl(shell, shell, "-c", scmd, (char *) NULL);
	se_errorf("execution of '%s' failed", scmd);
	exit(1);
  }
  
  while (wait((int *) 0) >= 0);
  
  intdel(0);
  set_modem_fio(fio);
}

/* 
 * execute an external command by forking 
 */

fork_exec_cmd(command)
	 char *command;
{
  kill_terminal();

  if (se_fork() == 0) {
	shell_exec(command);
	write_child_info(child_pipe, START_TERM, "Shell command finished");
	exit(0);
  }
}

/* 
 * start the terminal process
 */

start_terminal()
{
  if((term_pid = se_fork()) == 0) {
	terminal();
	exit(0);
  }
}

/* 
 * kill the terminal process
 */

kill_terminal()
{
  kill(term_pid, SIGTERM);
  while(wait((int *) 0) >= 0);	/* wait for the chold to die */
}

/* 
 * restart the terminal process (refresh) by killing it and starting
 * a new one
 */

restart_terminal()
{
  kill_terminal();
  start_terminal();
}

/* 
 * routines related to file handling 
 */

/* 
 * expand "~' in a file name
 */

char *expand_fname(fname, buffer)
	 char *fname, *buffer;
{
  char *home, *buf;
  short i;

  buf = buffer;

  for(i = 0; fname[i]; i++) {
	if (fname[i] == '~') {
	  if ((home = getenv("HOME")) == NULL)
		return NULL;
	  strcpy(buf, home);
	  buf += strlen(home);
	} 
	else {
	  *buf = fname[i];
	  buf++;
	}
  }
  *buf = '\0';

  return buffer;
}

/* 
 * open a file for reading by searching the current, then default, then
 * home directories
 */

FILE *open_file(name, fullname, directory)
	 char *name, *fullname, *directory;
{
  return open_file_va(name, fullname, directory, NULL);
}

/* 
 * similar to the above, but accepts more than one default directory
 */

FILE *open_file_va(name, fullname, dir1, dir2)
	 char *name, *fullname, *dir1, *dir2;
{
  FILE *fp;
  char *home;
  char buffer[REG_BUF];
 
  sprintf(fullname, "%s", name);

  if (fp = fopen(name, "r"))
	return fp;
  
  if(dir1) {
	sprintf(fullname, "%s/%s", expand_fname(SSpc(dir1), buffer), name);
	
	if (fp = fopen(fullname, "r"))
	  return fp;

	if(dir2) {
	  sprintf(fullname, "%s/%s", expand_fname(SSpc(dir2), buffer), name);
  
	  if (fp = fopen(fullname, "r"))
		return fp;
	}
  }

  if (home = (char *) getenv("HOME")) {
	sprintf(fullname, "%s/%s", home, name);
	
	if (fp = fopen(fullname, "r"))
	  return fp;
  } 

  se_warningf("could not open the file '%s'", name);
  se_warningf("tried the current directory");
  if(dir1) {
	se_warningf("tried the default directory '%s'", dir1);
	if(dir2)
	  se_warningf("tried the default directory '%s'", dir2);
  }
  se_warningf("tried the home directory '%s'", home);

  return NULL;
}

/* 
 * another implementation of the above using varargs, currently not used
 */

/*FILE *open_file_va(args)
	 va_list args;
	 va_decl
{
  FILE *fp;
  char *name, *dir, fullname[REG_BUF];
  char buffer[REG_BUF];  

  va_start(args);
  name = va_arg(args, char *);

  if (fp = fopen(name, "r"))
	return fp;

  while(dir = va_arg(args, char *))
	{
	  sprintf(fullname, "%s/%s", expand_fname(SSpc(dir), buffer), name);
  
	  if (fp = fopen(fullname, "r"))
		return fp;
	}

  va_end(args);

  if (dir = (char *) getenv("HOME")) {
	sprintf(fullname, "%s/%s", dir, name);
	
	if (fp = fopen(fullname, "r"))
	  return fp;
  } 

  showf("<< Seyon: file '%s' not in current, default, or home directory >>",
		name);
  return NULL;
}*/

/* 
 * read a file into a buffer
 */

read_file(fp, line)
	 FILE *fp;
	 char *line[];
{
  char buffer[REG_BUF + 1];
  int i;
  
  for (i=0; i < MAX_ENT && fgets(buffer, REG_BUF, fp) != NULL; i++)
	line[i] = strcpy((char*) malloc(sizeof(buffer)), SSpc(buffer));
  line[i] = NULL;
}

/* 
 * similar to the above, but closes the file after readsing it
 */

read_close_file(fp, line)
	 FILE *fp;
	 char *line[];
{
  read_file (fp, line);
  fclose(fp);
}

/* 
 * writes data to a pipe
 */

write_pipe_data(pd, data, size)
	 int *pd;
	 char *data;
	 int size;
{
  if (write(pd[1], data, size) == -1)
	show("<< Could not write to pipe >>");
}

/* 
 * reads data from a pipe
 */

read_pipe_data(pd, data, size)
	 int *pd;
	 char *data;
	 int size;
{
  read(pd[0], data, size);
}

/* 
 * not used
 */

/*char *get_word(str, word)
	 char *str, *word;
{
  char buf[WBSIZE], *wrd;

  strcpy(buf, str);
  wrd = str_parse(&buf);

  if(!wrd)
	word[0] = '\0';
  else {
	*buf = '\0';
	str
*/

