/*

======================================================================

  Seyon is Copyright (c) 1992 of Muhammad M. Saggaf. Seyon is not
  public domain. Permission is granted to use and distribute Seyon
  freely for any use and to sell it at any price without reference to
  the copyright owner provided that in all above cases Seyon is intact
  and is not made part of any program either in whole or in part and
  that this copyright notice is included with Seyon. Permission is
  also granted to modify the source as long as the modified source is
  not distributed.

======================================================================

*/

#include <stdio.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include "seyon.h"
#include "SeDecl.h"
#include "SeDeclX.h"

extern char *strsqtok();

struct child_info {
  short exit_status;
  char msg[80];
};

Boolean seyon_message_up = False;

void FunMessage(client_data)
	 XtPointer client_data;
{
  static short msg_index = 0;
  String msg;

  if (!seyon_message_up) {

	msg = qres.funMessages[msg_index++];
	if (msg == NULL) {
	  msg_index = 0;
	  msg = qres.funMessages[msg_index++];
	}

	SetStatusMessage(msg);
  }

  seyon_message_up = False;
  
  XtAppAddTimeOut((XtAppContext) client_data, 
				  qres.funMessagesInterval * 1000, 
				  FunMessage, client_data);
}

void MonitorChild(client_data)
	 XtPointer client_data;
{
  struct child_info child_info;

  read_pipe_data(child_pipe, &child_info, sizeof(child_info));

  if (*child_info.msg)
	SeyonMessage(child_info.msg);

  switch(child_info.exit_status) {
  case START_TERM:
	start_terminal();
	break;
  case START_AUTO_ZM:
	fork_exec_cmd(qres.zmodemAutoDownloadCommand);
	break;
  case SET_MESSAGE:
	break;
  case REMOVE_CURRENT_ITEM:
	RemoveCurrentItem();
	break;
  case DIAL_CIRCULATE:
	DialCirculate();
	break;
  case CLEANUP_AFTER_DIALER:
	DialerEnd();
	break;
  default:
	break;
  }
}

writef_child_info(pd, exit_status, a, b, c, d)
	 int *pd;
	 short exit_status;
	 char *a, *b, *c, *d;
{
  char buffer[SM_BUF];
  
  sprintf(buffer, a, b, c, d);
  write_child_info(pd, exit_status, buffer);
}

write_child_info(pd, exit_status, msg)
	 int *pd;
	 short exit_status;
	 char *msg;
{
  struct child_info child_info;

  child_info.exit_status = exit_status;

  if (msg)
	strcpy(child_info.msg, msg);
  else
	*child_info.msg = '\0';

  write_pipe_data(pd, &child_info, sizeof(child_info));
}

SeyonMessage(msg)
	 String msg;
{
  seyon_message_up = True;
  SetStatusMessage(msg);
}

read_seyon_file(name, fullname, line)
	 char *name, *fullname, *line[];
{
  FILE *fp;

  if ((fp = open_file(name, fullname, qres.defaultDirectory)) == NULL)
	return 0;

  read_file(fp, line);
  fclose(fp);

  return 1;
}

#define done(value, type)\
{\
  if (toVal->addr != NULL) {\
	if (toVal->size < sizeof(type)) {\
	  toVal->size = sizeof(type);\
	  return False;\
	}\
	*(type*)(toVal->addr) = (value);\
  }\
  else {\
    static type static_val;\
    static_val = (value);\
    toVal->addr = (XtPointer)&static_val;\
  }\
  toVal->size = sizeof(type);\
  return True;\
}
								   
Boolean CvtStringToStringArray(display, args, num_args, fromVal, toVal, 
							   converter_data)
	 Display *display;
	 XrmValue *args;
	 Cardinal *num_args;
	 XrmValue *fromVal;
	 XrmValue *toVal;
	 XtPointer *converter_data;
{
  String fromStr, buf, str;
  static String strArr[REG_BUF];
  short n;

  if (*num_args != 0)
	XtAppWarningMsg(app_con, "wrongParameters", "cvtStringToStringArray", 
					"XtToolkitError", 
					"String to StringArray conversion needs no extra arguments",
					(String*) NULL, (Cardinal*)NULL);
  
  fromStr = (String) fromVal->addr;
  buf = XtMalloc((strlen(fromStr)+1) * sizeof(char));
  strcpy(buf, fromStr);

  if ((strArr[0] = strsqtok(buf)) == NULL) {
	XtFree(buf);
	done(NULL, String*);
  }

  for(n = 1; n < XtNumber(strArr); n++)
	if ((strArr[n] = strsqtok(NULL)) == NULL)
	  done(strArr, String*);

  XtAppWarningMsg(app_con, "tooManyStrings", "cvtStringToStringArray", 
				  "XtToolkitError", 
				  "Too many strings specified for String to StringArray converter",
				  (String*) NULL, (Cardinal*) NULL);
  strArr[--n] = NULL;
  done(strArr, String*);
}

#undef done
