/*

======================================================================

  Seyon is Copyright (c) 1992 of Muhammad M. Saggaf. Seyon is not
  public domain. Permission is granted to use and distribute Seyon
  freely for any use and to sell it at any price without reference to
  the copyright owner provided that in all above cases Seyon is intact
  and is not made part of any program either in whole or in part and
  that this copyright notice is included with Seyon. Permission is
  also granted to modify the source as long as the modified source is
  not distributed.

======================================================================

*/

/* 
 * This file contains routines for Seyon's terminal. The main routine
 * is terminal(), which reads characters from the terminal and sends
 * them to the port. That routine also forks a child process that reads
 * characters from the port and writes them to the temrinal. Once the
 * parent receives SIGTERM (which should be sent by the grand parent), 
 * it kills the child and exits. NOTE: code is commented out is no longer
 * needed and will eventually be removed.
 */

#include <stdio.h>
#include <ctype.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include <X11/Intrinsic.h>

#include "seyon.h"
#include "SeDecl.h"
#include "SeDeclX.h"

extern FILE *tfp;				/* terminal pointer */
extern FILE *cfp;				/* capture file pointer */
extern Boolean capture;			/* are we capturing or not ? */

static jmp_buf  rtm;
int child_pid = 0;				/* ID of child process */

int child_pipe[2];
jmp_buf env_term;

void toggle(), cleanup();

int script_flag = 0;

void term_exit()
{
  /* jump to the terminal loop's check point */

  longjmp(env_term, 0);
}

/*
 * main routine. has two processes one to read from terminal and send 
 * to the port and the other to do the oppsite.
 */

void terminal()
{
  int c;

  script_flag = 0;		/* reset scripting flag */
/*  intdel(0);*/

/*  signal(SIGINT, SIG_IGN);
  signal(SIGQUIT, SIG_IGN);*/

  /* split into read and write processes */

  if((child_pid = se_fork()) == 0)
	port_to_tty();				/* child, read proc: read from port */
								/* and write to tty */
	
								/* parent, write proc: read from tty */
								/* and write to port */

  signal(SIGTERM, term_exit);	/* tell the program where to go when */
								/* SIGTERM is received */
  
  while(setjmp(env_term) == 0) { /* we want the program to come here from */
								/* the signal handler */
								/* loop until the signal is received */

	switch(c = coninp()) {		/* get a character from the terminal */
								/* and act upon it*/			   
#if 0
	case DIVCHAR:				/* divert a file through modem port */
	  divert();
	  break;
	  
/*	  goto fillicide;*/
#endif	  

	case '\n':					/* translate new line to carriage return */
	  if (qres.newlineTranslation) /* if newline translation mode is */
								/* in effect */
		c = '\r';
	  break;

	case 0x08:					/* translate backspace to delete */ 
	  if(qres.backspaceTranslation)	/* if del translation mode is */
  		c = 0x7f;				/*  in effect */
	  break;

	default:		
	  break;
	}

	if (qres.metaKeyTranslation && (c & 0x80)) { /* send ESC before the */
	  send_mbyte('\033');		/* character if meta key is pressed with */
	  send_mbyte(c & SBIT);		/* the  character and the meta key */
	}							/* translation mode is on */
	else
	  send_mbyte(c);			/* send the character to the port */
  } 

  signal(SIGTERM, SIG_IGN);		/* once the signal is received and we're */
								/* outside the loop, disable catching */
								/* the signal */

  kill(child_pid, SIGTERM);		/* kill him and wait for him to die, */
  while(wait((int *) 0) >= 0);	/* sounds vicious, doesn't it? the */
								/* child process is the read process from */
								/* the port */ 
}

/*
 * read proc: read from port and write to tty
 */

port_to_tty()
{
  int c;

  short auto_zm_flag = FALSE, auto_zm_index = 1;
  static char auto_zm_string[] = "**\030B00000000000000\r\212\021";

  signal(SIGTERM, cleanup);
  
  while(1){
	while((c = read_mbyte(0)) == -1);
	
	fputc(c, tfp);
	fflush(tfp);
	
	if (qres.zmodemAutoDownload) {
	  if (auto_zm_flag) {
		if ((c & SBIT) != (auto_zm_string[auto_zm_index++] & SBIT))
		  auto_zm_flag = FALSE;
		else if (auto_zm_index == strlen(auto_zm_string)) {
		  auto_zm_flag = FALSE;
		  se_notice("starting zmodem auto-download");
		  write_child_info(child_pipe, START_AUTO_ZM, 
						   "Starting zmodem auto-download");
		}
	  }
	  else if (c == auto_zm_string[0]) {
		auto_zm_flag = TRUE;
		auto_zm_index = 1;
	  }
	}
	
	if(capture && c != '\r')
	  fputc(c, cfp);
  }
  /* NOT REACHED */
}

/* cleanup, flush and exit
 */
void cleanup()
{
  fflush(tfp);
  exit(0);
}

/*
  Select a script file.  If the file exists, execute it, otherwise
  exit with a non-zero return.
  */

/*get_script()
{
  fprintf(tfp,"\r\nEnter script file: ");
  getsome(script_buf);
  if (script_buf[0] == '\0') {
	fprintf(tfp,"\r\n<<ABORTED>>\r\n");
	return 1;
  }
  if (access(script_buf, 04)) {
	fprintf(tfp," <<- NOT FOUND\r\n");
	return 1;
  }
  fprintf(tfp, "\r\n");
  linkflag = FALSE;
  script_file = script_buf;
  return 0;
}*/

/* Divert file into input stream, with optional delay after each newline.
 */
/*divert()
{
  FILE *fp;
  char buffer[NMSIZE];
  int c, i;
  
  fprintf(tfp,"\r\nFile? ");
  getsome(buffer);
  if((fp = fopen(buffer, "r")) == NULL){
	fprintf(tfp,"\r\nCan't open %s for input\r\n", buffer);
	return;
  }
  fprintf(tfp,"\r\nThe default delay of %d can be ", DRIBBLE);
  fprintf(tfp,"changed by typing the new delay after the 'y'\r\n");
  fprintf(tfp,"Delay after every newline (y/n)? ");
  
  getsome(buffer);
  fprintf(tfp,"\r\n");
  
  i = 0;
  if(mklow(buffer[0]) == 'y' && (i = atoi(buffer + 1)) == 0)
	i = DRIBBLE;
  while((c = getc(fp)) != EOF)
	if(c != '\n')
	  send_mbyte(c);
	else {
	  send_mbyte('\r');
	  if(i)
		sleep((unsigned) i);
	}
  fclose(fp);
}
*/
