/*

======================================================================

  Seyon is Copyright (c) 1992 of Muhammad M. Saggaf. Seyon is not
  public domain. Permission is granted to use and distribute Seyon
  freely for any use and to sell it at any price without reference to
  the copyright owner provided that in all above cases Seyon is intact
  and is not made part of any program either in whole or in part and
  that this copyright notice is included with Seyon. Permission is
  also granted to modify the source as long as the modified source is
  not distributed.

======================================================================

*/

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <X11/Xaw/Command.h>
#include <X11/Xaw/List.h>
#include "MultiList.h"
#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/Form.h>

#include "seyon.h"
#include "SeDeclX.h"

extern Widget statusMessage;

void SePopupMsg();
void SeEditSaveFile();

/* 
 * does nothing
 */

void DoNothing()
{
  return;
}

/* 
 * distroys a popup
 */

void DestroyParentPopup(widget, child)
	 Widget widget;
	 XtPointer child;
{
  XtDestroyWidget(XtParent((Widget) child));
}

/* 
 * same as the above
 */

void DestroyPopupPrompt(w, client_data, call_data)
	 Widget w;
	 XtPointer client_data, call_data;
{
  Widget popup = XtParent((Widget) client_data);
  XtDestroyWidget(popup);
} 

/* 
 * creates a button
 */

Widget SeAddButton(name, parent, call_back)
	 String name;
	 Widget parent;
	 void (*call_back)();
{
  Widget widget;

  widget = XtCreateManagedWidget(name, commandWidgetClass, parent, NULL, 0);
  XtAddCallback(widget, XtNcallback, call_back, (XtPointer) parent);
  return widget;
}

/* 
 * similar to the above, but also passes client data to the call back 
 */

Widget SeAddButtonWithClientData(name, parent, call_back, client_data)
	 String name;
	 Widget parent;
	 void (*call_back)();
	 XtPointer client_data;
{
  Widget widget;

  widget = XtCreateManagedWidget(name, commandWidgetClass, parent, NULL, 0);
  XtAddCallback(widget, XtNcallback, call_back, client_data);
  return widget;
}

/* 
 * creates a label
 */

Widget SeAddLabel(name, parent)
	 String name;
	 Widget parent;
{
  return XtCreateManagedWidget(name, labelWidgetClass, parent, NULL, 0);
}

/* 
 * sets a widget's label
 */

SeSetLabel(widget, label)
	 Widget widget;
	 String label;
{
  Arg args;

  XtSetArg(args, XtNlabel, label);
  XtSetValues(widget, &args, 1);
}

/* 
 * creates a toggle
 */

Widget SeAddToggle(name, parent, call_back)
	 String name;
	 Widget parent;
	 void (*call_back)();
{
  Widget widget;

  widget = XtCreateManagedWidget(name, toggleWidgetClass, parent, NULL, 0);
  if (call_back)
	XtAddCallback(widget, XtNcallback, call_back, (XtPointer) parent);
  return widget;
}

/* 
 * sets or unsets a toggle
 */

void SeSetUnsetToggle(widget, state)
	 Widget widget;
	 int state;
{
  Arg args;

  XtSetArg(args, XtNstate, state);
  XtSetValues(widget, &args, 1);
}

/* 
 * gets a toggle's state
 */

Boolean SeGetToggleState(widget)
	 Widget widget;
{
  Boolean state;
  Arg args;

  XtSetArg(args, XtNstate, &state);
  XtGetValues(widget, &args, 1);
  return state;
}

/* 
 * creates a box
 */

Widget SeAddBox(name, parent)
	 String name;
	 Widget parent;
{
  return XtCreateManagedWidget(name, boxWidgetClass, parent, NULL, 0);
}

/* 
 * creates a dialog
 */

Widget SeAddDialog(name, parent)
	 String name;
	 Widget parent;
{
  return XtCreateManagedWidget(name, dialogWidgetClass, parent, NULL, 0);
}

/* 
 * sets a dialog's label
 */

void SeSetDialogValue(dialog, value)
	 Widget dialog;
	 String value;
{
  Arg args;

  XtSetArg(args, XtNvalue, value);
  XtSetValues(dialog, &args, 1);
}  

/* 
 * creates a from
 */

Widget SeAddForm(name, parent)
	 String name;
	 Widget parent;
{
  return XtCreateManagedWidget(name, formWidgetClass, parent, NULL, 0);
}

/* 
 * sets a viewport's dimensions
 */

SeSetViewportDimensions(viewport, child, max_height)
	 Widget viewport, child;
	 Dimension max_height;
{
  SeSetWidgetWidth(viewport, SeWidgetWidth(child) + 14);

  if (SeWidgetHeight(child) > max_height)
	SeSetWidgetHeight(viewport, max_height);
}

/* 
 * sets a viwport's dimensions according to a child list
 */

SeSetViewportDimFromList(viewport, list, rows)
	 Widget viewport, list;
	 Cardinal rows;
{
  XFontStruct *font;
  Dimension   height, internalHeight, rowSpacing, borderWidth;
  Cardinal    n;
  Arg args[5];

  n = 0;
  XtSetArg(args[n], XtNfont, &font); n++;
  XtSetArg(args[n], XtNinternalHeight, &internalHeight); n++;
  XtSetArg(args[n], XtNrowSpacing, &rowSpacing); n++;
  XtSetArg(args[n], XtNborderWidth, &borderWidth); n++;
  XtGetValues(list, args, n);

  height = font->ascent + font->descent;
  height = height * rows + rowSpacing * (rows - 1) + 
	2 * (internalHeight + borderWidth);

  SeSetViewportDimensions(viewport, list, height);
}

/* 
 * sets a viwport's dimensions according to a child multi-list
 */

SeSetViewportDimFromMultiList(viewport, list, rows)
	 Widget viewport, list;
	 Cardinal rows;
{
  Dimension rowHeight, rowSpacing, borderWidth;
  Cardinal n;
  Arg args[2];

  n = 0;
  XtSetArg(args[n], XtNrowHeight, &rowHeight); n++;
  XtSetArg(args[n], XtNborderWidth, &borderWidth); n++;
  XtGetValues(list, args, n);

  SeSetViewportDimensions(viewport, list, 
						  rowHeight * rows + 2 * borderWidth);
}

/* 
 * creates a transient popup
 */

Widget SeCreatePopup(name, parent)
	 String name;
	 Widget parent;
{
  Arg     args[5];
  Position    x, y;
  Dimension   width, height, internalHeight;
  Cardinal    n;

  n = 0;
  XtSetArg(args[n], XtNwidth, &width); n++;
  XtSetArg(args[n], XtNheight, &height); n++;
  XtSetArg(args[n], XtNinternalHeight, &internalHeight); n++;
  XtGetValues(parent, args, n);
  XtTranslateCoords(parent, (Position) (width / 2),
					(Position) (height + 2 * internalHeight), &x, &y);
  
  n = 0;
  XtSetArg(args[n], XtNx, x);             n++;
  XtSetArg(args[n], XtNy, y);             n++;

  return XtCreatePopupShell(name, transientShellWidgetClass, 
							parent, args, n);
}

/* 
 * creates a top-level popup
 */

Widget SeCreateTopLevelPopup(name, parent, x_widget, y_widget)
	 String name;
	 Widget parent, x_widget, y_widget;
{
  Arg args[5];
  Position x, y, dummy;
  Dimension width, height, internalHeight;
  Cardinal n;


  if( x_widget == NULL)
	x_widget = parent;
  if( y_widget == NULL)
	y_widget = x_widget;

  n = 0;
  XtSetArg(args[n], XtNwidth, &width); n++;
  XtGetValues(x_widget, args, n);
  XtTranslateCoords(x_widget, (Position) (width / 2), 
					(Position) 0, &x, &dummy);
  
  n = 0;
  XtSetArg(args[n], XtNheight, &height); n++;
  XtGetValues(y_widget, args, n);
  XtTranslateCoords(y_widget, (Position) 0, (Position) height, 
					&dummy, &y);

/*
  XtSetArg(args[n], XtNinternalHeight, &internalHeight); n++;
  XtTranslateCoords(y_widget, (Position) 0, 
					(Position) (height + 2 * internalHeight), 
					&dummy, &y);
*/

  n = 0;
  XtSetArg(args[n], XtNx, x);             n++;
  XtSetArg(args[n], XtNy, y);             n++;
  
  return XtCreatePopupShell(name, topLevelShellWidgetClass, 
							parent, args, n);
}

/* 
 * pops up a message
 */

void SePopupMsg(parent, msg)
	 Widget parent;
	 String msg;
{
  Widget popup, dialog;
  Arg args;

  popup = SeCreatePopup("message", parent);

  XtSetArg(args, XtNlabel, msg);
  dialog = XtCreateManagedWidget("dialog", dialogWidgetClass, popup,
                                 &args, 1);

  XawDialogAddButton(dialog, "dismiss", DestroyPopupPrompt,
					 (XtPointer) dialog);

  XtPopup(popup, XtGrabExclusive);
}

void SePopupMsgf(parent, fmt, a, b, c)
	 Widget parent;
	 String fmt, a, b, c;
{
  Widget popup, dialog;
  Arg args;
  char buf[REG_BUF];

  sprintf(buf, fmt, a, b, c);
  SePopupMsg(parent, buf);
}  

void SePopupNotice(parent, pos_flag, title, call_back, msg)
	 Widget parent;
	 short pos_flag;
	 String title;
	 void (*call_back)();
	 String msg;
{
  Widget popup, dialog;
  Arg args[5];
  Cardinal n;

  n = 0;
  XtSetArg(args[n], XtNtitle, title); n++;

  popup = XtCreatePopupShell("notice", transientShellWidgetClass, 
							 parent, args, n);

/*  popup = SeCreatePopup("message", parent);*/

  n  = 0;
  XtSetArg(args[n], XtNlabel, msg); n++;
  
  dialog = XtCreateManagedWidget("dialog", dialogWidgetClass, popup,
								 args, n);

  XawDialogAddButton(dialog, "dismiss", call_back, (XtPointer) dialog);

  XtPopup(popup, XtGrabExclusive);
}

void SePopupNoticeF(parent, pos_flag, title, call_back, fmt, a, b, c, d)
	 Widget parent;
	 short pos_flag;
	 String title;
	 void (*call_back)();
	 String fmt, a, b, c, d;
{
  char buf[REG_BUF];

  sprintf(buf, fmt, a, b, c);
  SePopupNotice(parent, pos_flag, title, call_back, buf);
}

/* 
 * almost similar to the above
 */

void SeTransMsg(name, parent)
	 String name;
	 Widget parent;
{
  Widget popup, dialog;

  popup = SeCreatePopup(name, parent);
  dialog = XtCreateManagedWidget("dialog", dialogWidgetClass, popup,
								 NULL, 0);
  
  XawDialogAddButton(dialog, "dismiss", DestroyPopupPrompt,
					 (XtPointer) dialog);

  XtPopupSpringLoaded(popup);
}

/* 
 * pops up a message to the effect that a feature is not implemented
 */

void NotImplemented(w)
	 Widget w;
{
  SeTransMsg("notImplemented", w);
}

/* 
 * returns a widget's height
 */

Dimension SeWidgetHeight(widget)
	 Widget widget;
{
  Dimension height;
  Arg args;
  
  XtSetArg(args, XtNheight, &height);
  XtGetValues(widget, &args, 1);
  return height;
}

/* 
 * sets a widget's height
 */

SeSetWidgetHeight(widget, height)
	 Widget widget;
	 Dimension height;
{
  Arg args;
  
  XtSetArg(args, XtNheight, height);
  XtSetValues(widget, &args, 1);
}

/* 
 * returns a widget's width
 */

Dimension SeWidgetWidth(widget)
	 Widget widget;
{
  Dimension width;
  Arg args;
  
  XtSetArg(args, XtNwidth, &width);
  XtGetValues(widget, &args, 1);
  return width;
}

/* 
 * sets a widget's width
 */

SeSetWidgetWidth(widget, width)
	 Widget widget;
	 Dimension width;
{
  Arg args;
  
  XtSetArg(args, XtNwidth, width);
  XtSetValues(widget, &args, 1);
}

/* 
 * sets the status message
 */

void SetStatusMessage(msg)
	 String msg;
{
  SeSetLabel(statusMessage, msg);
/*  XFlush(XtDisplay(statusMessage));*/
}

/* 
 * similar to the above, but accepts a formmat string
 */

void SetStatusMessagef(a, b, c, d)
	 String a, b, c, d;
{
  char buffer[WBSIZE];

  sprintf(buffer, a, b, c, d);
  SetStatusMessage(buffer);
}

/* 
 * beeps the bell
 */

void SeBeep(widget)
	 Widget widget;
{
  XBell(XtDisplay(widget), 10);
}

/* 
 * displays a file in a popup
 */

void SeDisplayFile(parent, file_name)
	 Widget parent;
	 XtPointer file_name;
{
  Widget popup, form, text;
  Dimension width, height;
  Arg args[2];
  int n;

  popup = SeCreatePopup("display", parent);
  form = SeAddForm("form", popup);

  n = 0;
  XtSetArg(args[n], XtNtype, XawAsciiFile); n++;
  XtSetArg(args[n], XtNstring, (String) file_name); n++;

  text = XtCreateManagedWidget("text", asciiTextWidgetClass, form,
								 args, n);

  SeAddButton("dismiss", form, DestroyPopupPrompt);

  XtPopup(popup, XtGrabExclusive);
}

/* 
 * pops up a file for editing
 */

void SeEditFile(parent, file_name)
	 Widget parent;
	 XtPointer file_name;
{
  Widget popup, form, text;
  Dimension width, height;
  Arg args[2];
  int n;

  popup = SeCreatePopup("edit", parent);
  form = SeAddForm("form", popup);

  n = 0;
  XtSetArg(args[n], XtNtype, XawAsciiFile); n++;
  XtSetArg(args[n], XtNstring, (String) file_name); n++;

  text = XtCreateManagedWidget("text", asciiTextWidgetClass, form,
								 args, n);

  n = 0;
  XtSetArg(args[n], XtNwidth, &width); n++;
  XtSetArg(args[n], XtNheight, &height); n++;
  XtGetValues(text, args, n);

  SeAddButtonWithClientData("save", form, SeEditSaveFile, text);
  SeAddButton("dismiss", form, DestroyPopupPrompt);

  XtPopup(popup, XtGrabExclusive);
}

/* 
 * saves the file being edited
 */

void SeEditSaveFile(widget, text)
	 Widget widget;
	 XtPointer text;
{
  Widget textSource;
  Arg args;
  
  XtSetArg(args, XtNtextSource, &textSource);
  XtGetValues(text, &args, 1);

  if (XawAsciiSave(textSource) != True)
	SePopupMsg(widget, "File Save Failed");
}

