/*

======================================================================

  Seyon is Copyright (c) 1992 of Muhammad M. Saggaf. Seyon is not
  public domain. Permission is granted to use and distribute Seyon
  freely for any use and to sell it at any price without reference to
  the copyright owner provided that in all above cases Seyon is intact
  and is not made part of any program either in whole or in part and
  that this copyright notice is included with Seyon. Permission is
  also granted to modify the source as long as the modified source is
  not distributed.

======================================================================

*/

#include <stdio.h>
#include <signal.h>
#include <ctype.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>

#include "seyon.h"
#include "SeDecl.h"
#include "SeDeclX.h"

#ifdef HAVE_TERMIOS
#include <termios.h>
#else
#ifdef HAVE_TERMIO
#include <termio.h>
#endif
#endif

#define VERSION "1.5"
static char version[] = "$Version: 1.5 $";

jmp_buf erret;					/* non-loca-l error return */
pid_t term_pid = 0;				/* pid of the terminal subprocess */

#ifdef HAVE_TERMIOS
struct termios newmode, oldmode;
#else
#ifdef HAVE_TERMIO
struct termio newmode, oldmode;
#endif
#endif

void s_script(), s_set(), s_exit();
void hangup(), catch(), die(), cleanup_exit();

/*
    Command table
*/

static struct kw cmds[] = {
	{ "g",		s_script },
	{ "hangup",	hangup },
	{ "set",	s_set },
	{ "x",		s_exit },
	{ "exit",	s_exit },
/*	{ "!",		s_shell },
	{ "!!",		s_shell },
	{ "$",		s_shell },*/
	{ NULL,		NULL }};

int tfd;
FILE *tfp;

struct QueryResources qres;
XtAppContext app_con;

main(argc, argv)
int argc;
char *argv[];
{
  struct kw *ptr;
  int c;
  char *script = NULL;
  Widget topLevel, mainBox;

  static XrmOptionDescRec optionList[] =  {
	{ "-modem", ".modem", XrmoptionSepArg, (XPointer) NULL },
	{ "-script", NULL, XrmoptionSepArg, (XPointer) NULL },
  };

  if ((tfp = fopen("/dev/tty", "r+")) == NULL) {
	fprintf(stderr, "Cannot access local terminal!\n");
	exit(1);
  }
  
  tfd = fileno(tfp);
  
  io_get_attr(tfd, &oldmode);	/* get current console tty mode	*/
  newmode = oldmode;                  /* copy (structure) to newmode  */

  newmode.c_oflag &= ~OPOST;
  newmode.c_iflag &= ~(IXON | IXOFF);
  newmode.c_lflag &= ~(ICANON | ISIG | ECHO);
  newmode.c_cflag |= CREAD;
  newmode.c_cc[VMIN] = 1;
  newmode.c_cc[VTIME] = 1;
  
  setup_signal_handlers();

  topLevel = XtAppInitialize(&app_con, "Seyon", optionList, 
							 XtNumber(optionList), &argc, argv,
							 NULL, NULL, 0);

  get_resources(topLevel);

  if(argc > 1) {
	usage(argv[1]);
	exit(1);
  }
  
  if(qres.modem != NULL)
	mport(qres.modem);
							
  if (open_modem() < 0) {		/* open modem port and configure it */	
	se_error("modem initialization routine failed");
	se_error("no use continuing, press ENTER to exit");
	getchar();
	exit(1);
  }

  if (strcmp(VERSION, qres.version) != 0) {
	se_warning("missing or outdated app-defaults file");
	se_warning("program will not function properly");
  }

  init_variables();
  
  intdel(0);

  show( "\r\nSeyon Copyright (c) 1992 Muhammad M. Saggaf");
  showf("Version %s\n", VERSION);

  do_startup();
  if(qres.script != NULL)
	do_script(qres.script);

  create_command_center(app_con, topLevel);
  pipe(child_pipe);
  XtAppAddInput(app_con, child_pipe[0], (XtPointer) XtInputReadMask,
					 MonitorChild, NULL);

  start_terminal();

  XtRealizeWidget(topLevel);
  XtAppMainLoop(app_con);
}

usage(sw)
	 char *sw;
{
  char **ptr;
  char *babble[] = {
	"",
	"Usage:  seyon [-option ...] [-toolkitopion ...]",
	"",	
	"where options are",
	""
	"-modem device   Use 'device' as the modem port (eg, -modem /dev/tty00)",
	"-script file    Execute script file 'file' immediately",
	"",
	NULL };
  
  se_errorf("unknown or incomplete command-line switch: %s", sw);

  for (ptr = babble; *ptr != NULL; ptr++)
	show(*ptr);

  se_error("will not continue, press ENTER to exit");
  getchar();
}

setup_signal_handlers()
{
  signal(SIGINT, SIG_IGN);
  signal(SIGQUIT, SIG_IGN);
  signal(SIGCHLD, SIG_IGN);
  

#ifdef SIGBUS
  signal(SIGBUS, die);
#endif
  signal(SIGFPE, die);
  signal(SIGILL, die);
  signal(SIGIOT, die);
  signal(SIGSEGV, die);
  signal(SIGTERM, die);
  signal(SIGTRAP, die);
}

do_startup()
{
  FILE *startup_fp;
  char startupFile[REG_BUF];
  
  linkflag = 2;

  if ((startup_fp = open_file(qres.startupFile, startupFile, 
							  qres.defaultDirectory)) == NULL)
	return;

  exec_close_script(startup_fp);
}

void die(sig)
	 int sig;
{
  se_errorf("killed by signal %d",	sig);
  se_error("no use continuing, press ENTER to exit");
  getchar();

  cleanup_exit(1);
}

void cleanup_exit(status)
	 int status;
{
  show("<< Cleaning up >>");
  do_exit(status);
}

do_exit(rc)
{
  kill_terminal();

  unlock_tty();
  
  io_set_attr(tfd, &oldmode);
  io_flush(tfd);
  
  fclose(tfp);
  close_modem();
  
  XtDestroyApplicationContext(app_con);
  exit(rc);
}
    
void s_script()
{
    getword();

    if (word[0] == '\0') {
	fprintf(tfp, "Script file not specified\r\n");
	return;
    }

    do_script(word);
}

void s_exit(widget, client_data)
	 Widget widget;
	 XtPointer client_data;
{
  XtUnmapWidget((Widget) client_data);
  show("<< Bye >>");
  do_exit(0);
}


